
/*
 *    miracleshaper - Traffic shaper and prioritizer for DSLs.
 *    Copyright (C) 2007  Florian Zumbiehl <florz@florz.de>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#include <stdbool.h>
#include <time.h>

#include "config.h"

typedef long long schedtime_t;

struct event_timer {
	bool active;
	schedtime_t time;
	void (*callback)(void *);
	void *arg;
};

extern schedtime_t event_timer_hz;

void event_init(void);

static inline schedtime_t event_timer_now(){
#ifdef USE_KERNEL_CLOCK
	struct timespec ts;

	if(clock_gettime(CLOCK_MONOTONIC,&ts))die(true,"clock_gettime(CLOCK_MONOTONIC)");
	return ts.tv_sec*1000000000LL+ts.tv_nsec;
#else
	unsigned long long tsc;

	__asm__ __volatile__("rdtsc" : "=A" (tsc));
	return tsc;
#endif
}

void event_add_readfd(int,void (*)(int,void *),void *);

int event_timer_create(void (*)(void *),void *);

static inline void event_timer_advance(int timer,schedtime_t amount){
	extern struct event_timer *event_timers;

	if(event_timers[timer].time<event_timer_now())event_timers[timer].time=event_timer_now();
	event_timers[timer].time+=amount;
}

void event_timer_set_delay(int,schedtime_t);

static inline void event_timer_enable(int timer){
	extern struct event_timer *event_timers;

	event_timers[timer].active=true;
}

static inline void event_timer_disable(int timer){
	extern struct event_timer *event_timers;

	event_timers[timer].active=false;
}

void event_dispatch(void) __attribute__ ((noreturn));

